/*--------------------------------------------------------------------
 * 	Xilinx Virtex and Spartan JTAG programmer, based on
  	reference doc: XAPP 139 V1.6
	Table 7

	Rene van Leuken, April 2004

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

  p3j.c: Linux JTAG Xilinx Programmer - version 0.1

  Programs Xilinx FPGAs in JTAG mode through a parallel port,
  using a Xilinx Parallel Cable III/IV compatible interface by default.
  Input bitstream files must be in .bit (binary) format.
  The parallel port is used only in 'compatible' mode, so any old hardware will
  do. 
  No par port drivers required, better unload par port related modules,
  but must have r/w permissions for the parport device you want to
  use.  Typical you need to be the root user.

  To compile this code use something like:  gcc -Wall -O2 -o p3j p3j.c

  Jan Panteltje 2006 <panteltje @ yahoo . com> // remove spaces

  Based on original work by:
  Rudolf Usselmann	ASICS World Services	rudi(at)asics(dot)ws

  who based on original work by:
  Reinoud <reinoud@remove.et.tudelft.nl>

  --------------------------------------------------------------------
  (c) 2006 Jan Panteltje
  		Changed ioctls to direct io, so no par port modules needed, user must be root.
  		Added tst for DONE, added par port address input.
  		Clean compile with -Wall on gcc-2.95, gcc-3.3, gcc-4.0.
  (c) 2003/2004 Rudolf Usselmann
  (c) 2001 Reinoud Lamberts

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  USA

--------------------------------------------------------------------*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/io.h>
#include <getopt.h>
#include <ctype.h>


/*--- interface configuration ---*/

/* Configure the interface/cable with the defines below.  Xilinx
   parallel cable compatible is the default.  If your interface
   doesn't use a signal, simply make the corresponding mask all zero.

   Note that the mask defines below are named after FPGA signals they
   control, and not after names used in the Xilinx parallel cable
   schematic.  This is because the Xilinx cable is not the only
   target, and the names it uses are confusing. */


#define PVERSION			"0.2"
#define DEFAULT_PARPORT		0x378


// output signal masks 

#define O_ENABLE 0x10		// Enable TDI, TCK and TSM outputs
#define O_START  0x10		// Pulses TDO (DONE Input to Par. Port) (UNUSED)

#define O_TDI_0	0x00		// TDI Low
#define O_TDI_1	0x01		// TDI High

#define O_TCK_0	0x00		// TCK Low
#define	O_TCK_1	0x02		// TCK High

#define O_TMS_0	0x00		// TMS Low
#define O_TMS_1	0x04		// TMS High

// input signal masks; for *_OK give masked expected OK value 
#define I_SENSE   	 PARPORT_STATUS_ERROR
#define I_SENSE_OK	 I_SENSE
#define I_DONE   	  PARPORT_STATUS_SELECT
#define I_DONE_OK 	 I_DONE

// JTAG Commands
#define CMD_CFG		0x05
#define CMD_JSTART	0x0c
#define CMD_IDCODE	0x09

unsigned char byte;
int parport_data_address, parport_status_address;


char *strsave(char *s) /* save string s somewhere */
{
char *p;

p = (char *) malloc(strlen(s) + 1);
if(!p) return 0;

strcpy(p, s);
return p;
} /* end function strsave */


// Read TDO pin
unsigned char read_tdo()
{
unsigned char stat;		/* status (input) data from parport */

stat = inb(parport_status_address);

// !SELECT is bit 4 status port, test bit 4.
if( (stat >> 4) & 1) return 1;

return 0;
} /* end function read_tdo */ 


// Read Vcc sense pin
unsigned char read_vcc()
{
unsigned char stat;		/* status (input) data from parport */

stat = inb(parport_status_address);

if( (stat >> 3) & 1)  return 1;

return 0;
} /* end fucntion read_vcc */


// Send a single bit to the device (1 clock cycle). 'val' is the
// value for TDI and tms is the value for the TMS pin.
void send_bit(unsigned char val, unsigned char tms)
{
unsigned char out;

out = O_TCK_0 | O_ENABLE;
out |= (tms == 0) ? O_TMS_0 : O_TMS_1;
out |= (val == 0) ? O_TDI_0 : O_TDI_1;

outb(out, parport_data_address);

out = O_TCK_1 | O_ENABLE;
out |= (tms == 0) ? O_TMS_0 : O_TMS_1;
out |= (val == 0) ? O_TDI_0 : O_TDI_1;

outb(out, parport_data_address);	

} /* end function send_bit */

 

// Send an entire byte to the device. If last is '1', TMS will
// be high for the last bit transmitteda (e.g. force trasnition
// to EXIT1 state).
void send_data_byte(unsigned char val, unsigned char last)
{
int n;

for(n = 7; n >= 0; n--)
	{
	send_bit( (val >> n) & 0x01, ( (n == 0) & last ) );
	}
} /* end function send_data_byte */


// Identical to send_data_byte, but also reads TDO. Created seperate
// subroutines for write only and write & read mainly for speed-up
// reasons.
void read_data_byte(unsigned char val, unsigned char last)
{
int n;
unsigned char bit;

byte = 0;
for(n = 7; n >= 0; n--)
	{
	send_bit((val >> n) & 0x01, ( (n == 0) & last ) );
	bit = read_tdo();
	byte |= bit << (7 - n);
	}
} /* end function read_data_byte */


void decode_id(int id)
{
int manuf, size, family, rev;

if((id & 0x01) != 1)
	{
	fprintf(stderr, "Warning: Bit 0 (LSB) of device id is not '1' !\n");
	}

manuf = (id >> 01) & 0x07ff;
size = (id >> 12) & 0x01ff;
family = (id >> 21) & 0x007f;
rev = (id >> 28) & 0x000f;

fprintf(stderr, "Device ID: %x\n", id);
fprintf(stderr, "Manuf: %x, Part Size: %x, Family Code: %x, Revision: %0d\n", manuf,  size, family, rev);

if(manuf != 0x49)
	{
	fprintf(stderr, "I can only program Xilinx Virtex and Spartan devices\n");
	}
} /* end function decode_id */


void print_usage()
{
fprintf(stderr, "\
Usage: p3j [-h] [-v] [-p par_port_address] bitfile\n\n\
-h        help (this help)\n\
-p int    par port address in dec or hex, default is 0x%03x\n\
-v        debug (prints functions and arguments)\n\
", DEFAULT_PARPORT);

} /* end function print_usage */



int main(int argc, char *argv[])
{
int a, i;
char *rbname = 0;		/* rbt file name */
FILE *f;				/* rbt file ptr */
char c;					/* char from f */
unsigned char uc;
long bitcount;			/* bits transferred */
int n, x, len, token;
int devid;
int skip_argument_flag;
int debug_flag;

// ====================================================
// ===== get arguments

/* defaults */
debug_flag  = 0;
parport_data_address = DEFAULT_PARPORT;
parport_status_address = parport_data_address + 1;
/* end defaults */

while(1)
	{
	a = getopt(argc, argv, "hp:v");
	if(a == -1) break;

	switch(a)
		{
		case 'h':
			print_usage();
			exit(1);
			break;
		case 'p':
			sscanf(optarg, "%i", &parport_data_address);
			break;
		case 'v':
			debug_flag = 1;
			break;
        case -1:
        	break;
		case '?':
			if(isprint(optopt) )
 				{
 				fprintf(stderr, "Unknown option `-%c'.\n", optopt);
 				}
			else
				{
				fprintf(stderr, "Unknown option character `\\x%x'.\n", optopt);
				}
			print_usage();

			exit(1);
			break;			
		default:
			print_usage();

			exit(1);
			break;
		}/* end switch a */
	}/* end while getopt() */

skip_argument_flag = 0;
for(i = 0; i < argc - 1; i++)
	{
	if(argv[i + 1][0] == '-')
		{
		skip_argument_flag = 1;
		continue;
		}

	if(! skip_argument_flag)
		{
		rbname = strsave(argv[1 + i]);
		if(! rbname)
			{
			fprintf(stderr, "Could not allocate space for %s, aborting.\n", rbname);

			exit(1);
			}
 
		break;
		} /* end if ! command line flag */

	skip_argument_flag = 0;
	} /* end while al input files */

fprintf(stderr, "Panteltje p3j-%s\n", PVERSION);

if(! rbname)
	{
	fprintf(stderr, "p3j: no bitfilename specified, use p3j -h for help, aborting.\n");

	exit(1);
	}

/* allow io to par port (as root) */
iopl(3);

// ====================================================
// ===== get input file

f = fopen(rbname, "r");
if(!f)
	{
	fprintf(stderr, "p3j: could not open bitstream file %s for read, aborting.\n", rbname); 

	exit(1);
	}
	
	
// ====================================================
// =====        Parse binary Header             PLEASE REWRITE THIS !

uc = getc(f);
len = uc << 8;
uc = getc(f);
len += uc;
for(x = 0; x < len; x++) c = getc(f);

uc = getc(f);
len = uc << 8;
uc = getc(f);
len += uc;
//fprintf(stderr, "Length: %0d\n",len);

uc = getc(f);
token = uc;
//fprintf(stderr, "Token: %x\n",token);

uc = getc(f);
len = uc << 8;
uc = getc(f);
len += uc;
//fprintf(stderr, "Length: %0d\n",len);

fprintf(stderr, "Design Name: ");

for(x = 0; x < len; x++)
	{
	uc = getc(f);
	fprintf(stderr, "%c", uc);
    }
fprintf(stderr, "\n");

uc = getc(f);
token = uc;
//fprintf(stderr, "Token: %x\n",token);

uc = getc(f);
len = uc << 8;
uc = getc(f);
len += uc;
//fprintf(stderr, "Length: %0d\n",len);

fprintf(stderr, "Device: ");

for(x = 0; x < len; x++)
	{
	uc = getc(f);
	fprintf(stderr, "%c", uc);
    }
fprintf(stderr, "\n");

uc = getc(f);
token = uc;
//fprintf(stderr, "Token: %x\n",token);

uc = getc(f);
len = uc << 8;
uc = getc(f);
len += uc;
//fprintf(stderr, "Length: %0d\n",len);

fprintf(stderr, "Date: ");

for(x = 0; x < len; x++)
	{
	uc = getc(f);
	fprintf(stderr, "%c", uc);
	}
fprintf(stderr, "\n");

uc = getc(f);
token = uc;
//fprintf(stderr, "Token: %x\n",token);

uc = getc(f);
len = uc << 8;
uc = getc(f);
len += uc;
//fprintf(stderr, "Length: %0d\n",len);

fprintf(stderr, "Time: ");

for(x = 0; x < len; x++)
	{
	uc = getc(f);
	fprintf(stderr, "%c", uc);
	}
  fprintf(stderr, "\n");

uc = getc(f);
token = uc;
//fprintf(stderr, "Token: %x\n",token);

uc = getc(f);
len = uc << 24;
uc = getc(f);
len += uc << 16;
uc = getc(f);
len += uc << 8;
uc = getc(f);
len += uc;
fprintf(stderr, "Bitstream Length: %0d bits\n", len * 8);


// ====================================================
// ===== check power 
if(! read_vcc() )
	{
	fprintf(stderr, "ERROR: no power on interface\n");

	exit(1);
	}

// ====================================================
// ##########  Start of JTAG Programming sequence

// ====================================================
// ##########  RESET Test Logic

for(n = 0; n < 6; n++)  send_bit(0, 1);		// Test Logic Reset  // 1

// tdi tms
send_bit(0, 0);		// 2

// State = IDLE

send_bit(0, 1);		// 3
send_bit(0, 1);		// 3
send_bit(0, 0);		// 4
send_bit(0, 0);		// 4

// ====================================================
// ##########  IDCODE Instruction
send_bit(1, 0);		// 
send_bit(0, 0);		// 
send_bit(0, 0);		// 
send_bit(1, 0);		// 
send_bit(0, 1);		// 


send_bit(0, 1);		//
send_bit(0, 1);		//

send_bit(0, 0);		//
send_bit(0, 0);		//

read_data_byte(0x00, 0);
devid = byte;

read_data_byte(0x00, 0);
devid += byte << 8;

read_data_byte(0x00, 0);
devid += byte << 16;

read_data_byte(0x00, 1);
devid += byte << 24;

decode_id(devid);

send_bit(0, 1);		//

send_bit(0, 1);		//
send_bit(0, 1);		//
send_bit(0, 0);		//
send_bit(0, 0);		//

// END IDCODE


// ====================================================
// ##########  CONFIGURE
// CFG_IN 0101
send_bit(1, 0);		// 5
send_bit(0, 0);		// 5
send_bit(1, 0);		// 5
send_bit(0, 0);		// 5

send_bit(0, 1);		// 6

send_bit(0, 1);		// 7
send_bit(0, 1);		// 7
send_bit(0, 0);		// 8
send_bit(0, 0);		// 8

// ====================================================
// SEND DATA

bitcount = 0;
fprintf(stderr, "\nProgramming ... ");

for(n = 0; n < len; n++)
	{
	uc = getc(f);
	send_data_byte(uc, n == (len - 1));
	bitcount += 8;

	if((bitcount % (256 * 1024)) == 0) putc('*', stderr);
	}

fprintf(stderr, "\nProgrammed %ld bits", bitcount);

send_bit(0, 1);		// 11
send_bit(0, 1);		// 12
send_bit(0, 1);		// 12
send_bit(0, 0);		// 13
send_bit(0, 0);		// 13

// ====================================================
// ##########  JSTART 1100

send_bit(0, 0);		// 14
send_bit(0, 0);		// 14
send_bit(1, 0);		// 14
send_bit(1, 0);		// 14

send_bit(0, 1);		// 15
send_bit(0, 1);		// 16
send_bit(0, 1);		// 16

for(n = 0; n < 17; n++) send_bit(0, 0);		// Clock-in RUN-TEST sequence

send_bit(0, 1);		// 18
send_bit(0, 1);		// 18
send_bit(0, 0);		// 19
send_bit(0, 0);		// 19

// DONE !
// ====================================================

fprintf(stderr, "\n");

/* test if DONE */
uc = read_tdo();
fprintf(stderr, "Ready, DONE=%d\n", uc);

exit(0);
} /* end function main */



